/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_fixts.c: Allows Admins to change(fix) Channel TS.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  m_fixts.c - (C) Alan 'alz' Milford, 8th May 2003. (alz@froginternet.com)
 *
 *  Warning: This is a NETWORK-WIDE module, its either all or nothing,
 *           otherwise you will break things quite horribly and cause all
 *           sorts of weird things to happen.
 *
 *  Usage Details:
 * 
 *  This command is available for use by server admins (+a flag),
 * 
 *  /chkts #channel - Replies with #channel's current TS.
 *  /fixts #channel <NEW TS> - Changes #channel's TS to the value stated.
 *
 *  "/fixts" will only change a channel's TS if:
 *  1) The current channel TS is more than 2 years old.
 *  2) The new TS value is less than 2 weeks old.
 *  3) The new TS value is NOT in the future.
 *  4) The new TS value is NOT before the channel's existing TS.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "list.h"
#define CHANMINAGE 62899200
#define CHANMAXAGE 1209600

static void mo_fixts(struct Client *, struct Client *, int, char**);
static void ms_sfixts(struct Client *, struct Client *, int, char**);
static void mo_chkts(struct Client *, struct Client *, int, char**);

struct Message fixts_msgtab = {
  "FIXTS", 0, 0, 3, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_fixts}
};

struct Message sfixts_msgtab = {
  "SFIXTS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_sfixts, m_ignore}
};

struct Message chkts_msgtab = {
  "CHKTS", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_chkts}
};
#ifndef STATIC_MODULES

void
_modinit(void)
{
  mod_add_cmd(&fixts_msgtab);
  mod_add_cmd(&sfixts_msgtab);
  mod_add_cmd(&chkts_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&fixts_msgtab);
  mod_del_cmd(&sfixts_msgtab);
  mod_del_cmd(&chkts_msgtab);
}

const char *_version = "3.0";
#endif


/*
** mo_fixts
**      parv[0] = sender prefix
**      parv[1] = channel
**      parv[2] = ts to change to
*/
static void mo_fixts(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
  struct Channel *chptr;
  int newts = 0;
  int oldts = 0;
  if (!IsOperAdmin(source_p))
    {
      sendto_one(source_p, ":%s NOTICE %s :You have no A flag", me.name, parv[0]);
      return;
    }
  chptr= hash_find_channel(parv[1]);
  if( chptr == NULL )
    {
      sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
		 me.name, parv[0], parv[1]);
      return;
    }
  if (parv[2] == NULL)
    {
      sendto_one(source_p, ":%s NOTICE %s :No new TS specified", me.name, parv[0]);
      return;
    }
  oldts = chptr->channelts;
  newts = atoi(parv[2]);
  if (newts > CurrentTime) 
    {
     sendto_wallops_flags(FLAGS_WALLOP, &me,
              "Failed TS Change (Future Time): %s from %d to %d by %s!%s@%s",
              parv[1], oldts, newts, source_p->name, source_p->username, source_p->host);        
    } 
  else if (newts < chptr->channelts)
    {
     sendto_wallops_flags(FLAGS_WALLOP, &me,
              "Failed TS Change (Before Creation): %s from %d to %d by %s!%s@%s",
              parv[1], oldts, newts, source_p->name, source_p->username, source_p->host);  
    } 
  else if (newts < CHANMAXAGE)
    {
     sendto_wallops_flags(FLAGS_WALLOP, &me,
              "Failed TS Change (New TS Too Old): %s from %d to %d by %s!%s@%s",
              parv[1], oldts, newts, source_p->name, source_p->username, source_p->host);  
    } 
  else if (chptr->channelts > (CurrentTime - CHANMINAGE)) 
    {
     sendto_wallops_flags(FLAGS_WALLOP, &me,
              "Failed TS Change (Channel Too New): %s from %d to %d by %s!%s@%s",
              parv[1], oldts, newts, source_p->name, source_p->username, source_p->host);
    } 
  else 
    {
     chptr->channelts = newts;
     sendto_server(client_p, source_p, chptr, NOCAPS, NOCAPS,
                        LL_ICLIENT,
                        ":%s SFIXTS %s %d %s!%s@%s",
                        me.name, parv[1], newts, source_p->name, source_p->username, source_p->host);
     sendto_wallops_flags(FLAGS_WALLOP, &me,
              "TS CHANGE: %s %d -> %d [%s!%s@%s]",
              parv[1], oldts, newts, source_p->name, source_p->username, source_p->host);
    }
}

/*
** mo_chkts
**      parv[0] = sender prefix
**      parv[1] = channel
*/
static void mo_chkts(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
  struct Channel *chptr;
  if (!IsOperAdmin(source_p))
    {
      sendto_one(source_p, ":%s NOTICE %s :You have no A flag", me.name, parv[0]);
      return;
    }
  chptr= hash_find_channel(parv[1]);
  if( chptr == NULL )
    {
      sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
		 me.name, parv[0], parv[1]);
      return;
    }
  sendto_one(source_p, ":%s NOTICE %s :Channel %s TS is %ld", me.name, parv[0], parv[1], chptr->channelts);
}

/*
 * ms_sfixts
 *
 * inputs	-
 * output	- none
 * side effects	- handles TS fixes from remote servers.
 *
 */
static void ms_sfixts(struct Client *client_p, struct Client *source_p,
	                 int parc, char *parv[])
{
  struct Channel *chptr;
  int newts = 0;
  int oldts = 0;
  chptr= hash_find_channel(parv[1]);
  if ( chptr != NULL )
  {
   oldts = chptr->channelts;
   newts = atoi(parv[2]);
   chptr->channelts = newts;
   sendto_wallops_flags(FLAGS_WALLOP, &me,
              "TS CHANGE: %s %d -> %d [%s]",
               parv[1], oldts, newts, parv[3]);
   sendto_server(source_p, NULL, chptr, CAP_TBURST, NOCAPS, NOFLAGS,
		":%s SFIXTS %s %d %s!%s@%s",
		source_p->name, parv[1], newts, source_p->name, source_p->username, source_p->host);
  }
}

